import abc

class TaskListener(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def onTaskUpdate(self, task):
        raise NotImplementedError()

class Task():
    def __init__(self, listener, task_name, task_run_id):
        self.listener = listener

        self.task_name = task_name
        self.task_run_id = task_run_id

        self.task_status = None
        self.task_input = None
        self.task_output = None
        self.task_metadata = None

    def update(self, task_status=None, task_input=None, task_output=None, task_metadata=None):
        if task_status is not None:
            self.task_status = task_status
        if task_input is not None:
            self.task_input = task_input
        if task_output is not None:
            self.task_output = task_output
        if task_metadata is not None:
            self.task_metadata = task_metadata

        self.listener.onTaskUpdate(self)

        return self



import __main__
import os
import request

ENV_ECS_CONTAINER_METADATA_URI = 'ECS_CONTAINER_METADATA_URI'

def isECS():
    return os.environ(ENV_ECS_CONTAINER_METADATA_URI, None) is not None

def getTaskMetadataFromECS():
    res = requests.get(os.getenv('ECS_CONTAINER_METADATA_URI'))
    ecs_container_metadata = res.json()

    # TODO: There's no way this is stable:
    task_definition_family = ecs_container_metadata['Labels']['com.amazonaws.ecs.task-definition-family']
    container_name = ecs_container_metadata['Labels']['com.amazonaws.ecs.container-name']
    task_id = ecs_container_metadata['Labels']['com.amazonaws.ecs.task-arn'].split('/')[-1]

    log_group_name = '/ecs/' + task_definition_family
    log_stream_name = 'ecs/' + container_name + '/' + task_id

    return dict(
        logGroupName=log_group_name,
        logStreamName=log_stream_name,
        region=os.getenv('AWS_REGION', None),
        execution_env=os.getenv('AWS_EXECUTION_ENV', None),
        ecs_container_metadata=ecs_container_metadata
    )

def wrapMainFunction(dagger_api, func, task_name=None, task_run_id=None):
    if task_name is None:
        task_name = __main__.__file__

    def _wrapper(*args, **kwargs):
        task = None
        try:
            task_input = dict(
                args=*args,
                kwargs=**kwargs
            )

            task_metadata = dict(
                # logGroupName=context.log_group_name,
                # logStreamName=context.log_stream_name,
                region=os.getenv('AWS_REGION', None),
                execution_env=os.getenv('AWS_EXECUTION_ENV', None)
            )

            task = api.createTask(
                task_name,
                task_run_id,
                task_input=task_input,
                task_metadata=task_metadata
            )
        except Exception as e:
            print('Failed to initialize dagger')
            print(e)

        try:
            task_output = func(*args, **kwargs)
        except Exception as e:
            if task is not None:
                task.update(task_status='failed', task_output=str(e))

        try:
            task.update(task_status='succeeded', task_output=task_output)
        except Exception as e:
            print('Failed to log dagger output')
            print(e)

        return task_output

# {"function_version":"$LATEST","function_name":"complicated_job_rss","memoryLimitInMB":"512","logGroupName":"/aws/lambda/complicated_job_rss","logStreamName":"2020/05/24/[$LATEST]e47d9a855816499abffb1616ac6c1e9b","invokedFunctionArn":"arn:aws:lambda:us-east-2:985550760282:function:complicated_job_rss","awsRequestId":"00080e00-c853-469f-9ad8-57dca5c6c014"}
# {
#     'DockerId': 'd0e11205a44b5768cb03f9e160f2f4afb2b378d554974c42d018d262c2cfb7d8',
#     'Name': 'doralgo-l53-support', 
#     'DockerName': 'ecs-doralgo-l53-support-staging-3-doralgo-l53-support-a4df94f4ca8b88e1b101', 
#     'Image': '340301842845.dkr.ecr.us-west-2.amazonaws.com/doralgo-l53-support:staging', 
#     'ImageID': 'sha256:ab15381f4b91caeb9252181e2193d3d57099bd2c77161bbf22ce34960ade7ec2', 
#     'Labels': {'com.amazonaws.ecs.cluster': 'arn:aws:ecs:us-west-2:340301842845:cluster/doralgo', 'com.amazonaws.ecs.container-name': 'doralgo-l53-support', 'com.amazonaws.ecs.task-arn': 'arn:aws:ecs:us-west-2:340301842845:task/013a7a37-5e6f-4623-98ed-539340ef7711', 'com.amazonaws.ecs.task-definition-family': 'doralgo-l53-support-staging', 'com.amazonaws.ecs.task-definition-version': '3'}, 
#     'DesiredStatus': 'RUNNING', 
#     'KnownStatus': 'RUNNING', 
#     'Limits': {'CPU': 0, 'Memory': 2048}, 
#     'CreatedAt': '2020-06-07T16:07:45.321115096Z', 
#     'StartedAt': '2020-06-07T16:08:10.522444681Z', 
#     'Type': 'NORMAL', 
#     'Networks': [{'NetworkMode': 'awsvpc', 'IPv4Addresses': ['10.0.21.154']}]
# }


import os

def isLambda():
    return os.getenv('_HANDLER', None) is not None

def init(api):
    import bootstrap

    old_handle_request = bootstrap.handle_event_request
    def _newHandleRequest(lambda_runtime_client, request_handler, *args):
        def _newRequestHandler(event, context):
            task_input = event

            task = api.createTask(
                context.function_name, 
                context.aws_request_id, 
                task_input=task_input,
                task_metadata=dict(
                    logGroupName=context.log_group_name,
                    logStreamName=context.log_stream_name,
                    region=os.getenv('AWS_REGION', None)
                )
            )

            try:
                task_output = request_handler(event, context)
            except Exception as e:
                task.update(task_status='failed', task_output=str(e))
                raise e

            task.update(task_status='succeeded', task_output=task_output)
            return task_output

        old_handle_request(lambda_runtime_client, _newRequestHandler, *args)
    bootstrap.handle_event_request = _newHandleRequest



import os

from dagger.DaggerAPI import DaggerAPI
from dagger.integrations.aws_lambda import init as initLambda, isLambda
import dagger.integrations.aws_ecs

def initDagger(api_key, auto_initialize=True):
    api = DaggerAPI(api_key)

    if auto_initialize:
        if isLambda():
            initLambda(api)

    return api



# Dagger on Lambda
from dagger import initDagger
initDagger('asdf')

# Dagger in Python CLI Runtime
from dagger import initDagger
dagger_api = initDagger('asdf')

def main(input):
    return 1

if __name__ == "__main__":
    main = dagger_api.wrapTaskFunction(main)
    main()

# Dagger manually
from dagger import initDagger
dagger_api = initDagger('asdf')

def main(some_input):
    dagger_task = dagger_api.createTask('my task name', 'my unique run id')
    dagger_task.update(
        status='running',
        input=some_input
    )

    # do some things

    dagger_task.update(
        status='succeeded',
        output=some_output_value
    )
